//! event ch.sort-lines-ascending: sortLinesAscending
//! event ch.sort-lines-descending: sortLinesDescending
//! event ch.randomize-lines: randomizeLines
//! event ch.reverse-lines: reverseLines
//! event ch.unique-lines: uniqueLines
//! event ch.sort-ascending-and-unique-lines: sortAscendingAndUniqueLines

var sortLinesAscending = function() { sortLines(1); }
function sortLinesDescending() { sortLines(-1); }
function sortLines(direction, shouldUnique) {
   processLines(function(lineContents) {
      lineContents.sort(function(left, right) {
         
         var left = NSString.stringWithString_(left);
         var right = NSString.stringWithString_(right);
         
         return direction * left.localizedStandardCompare_(right);
         
      });
      
      if (shouldUnique === true) {
         var lastObject = null;
         var result = [];
         lineContents.each(function(x) {
            if (x !== lastObject) {
               result.push(x);
            }
            lastObject = x;
         });
         
         lineContents = result;
      }
      
      return lineContents;
   });
}

function randomizeLines() {
   processLines(function(lineContents) {
      return lineContents.randomize();
   });
}

function reverseLines() {
   processLines(function(lineContents) {
      lineContents.reverse();
      return lineContents;
   });
}

function uniqueLines() {
   processLines(function(lineContents) {
      return lineContents.unique();
   });
}

function sortAscendingAndUniqueLines() {
   sortLines(1, true);
}

function processLines(mapping) {
   Recipe.run(function(rctx) {
      
      // Get the range of the selected line(s)
      var lineRange = rctx.contentRangeOfLinesInRange(rctx.selection);
      var lineContents = [];
      
      // Look through each line. Add its contents to the array
      rctx.foreachLine(function(lctx) {
           
           var currentLine = rctx.textInRange(lctx.contentRange);
           lineContents.push("" + currentLine);
      }, lineRange);
      
      
      // Process the lines
      lineContents = mapping(lineContents);
            
      // Replace each line with its sorted version
      var i = 0;
      var offset = rctx.foreachLine(function(lctx) {
         
         if (i >= lineContents.length) {
            i++;
            return false;
         }
         
         return lineContents[i++];
         
      }, lineRange);
   });
}