//! event ch.comment-line: comment_line
//! event ch.comment-block: comment_block

function comment_line_inner(lcom) {
   Recipe.run(function(rctx) {
      
      // Get the range of the selected line(s)
      var lineRange = rctx.contentRangeOfLinesInRange(rctx.selection);
      
      var oldSelection = rctx.selection;
      
      var allcom = true;
      var anycom = false;
      
      var startIndentation = null;
      
      // Look through each line. Does it start with a comment?
      rctx.foreachLine(function(lctx) {
           var currentLine = rctx.textInRange(lctx.contentRange);
           // sputs("currentLine: " + currentLine);
           
           var ind = indentation(currentLine);
           if (currentLine.trim().length > 0 && (startIndentation === null || ind.length < startIndentation.length))
              startIndentation = ind;
           
           currentLine = currentLine.trim();
           var istruecom = currentLine.startsWith(lcom);
           var iscom = currentLine.length == 0 || istruecom;
           anycom |= istruecom;
           if (!iscom)
              allcom = false;
        }, lineRange);
      
      if (startIndentation === null) {
         startIndentation = "";
      }
      
      var runningdelta = 0;
      var selrange = rctx.selection;
      var newselrange = rctx.selection;
      
      var offset = rctx.foreachLine(function(lctx) {
         
         var rval = null;
         if (allcom && anycom) {
            // Do uncommenting
            var currentLine = rctx.textInRange(lctx.contentRange);
            var ind = indentation(currentLine);
            var newline = currentLine.trimLeft();
            if (newline.startsWith(lcom + " "))
               newline = "" + ind + newline.substr(lcom.length + 1);
             else if (newline.startsWith(lcom))
               newline = "" + ind + newline.substr(lcom.length);
            rval = newline;
         }
         else {
            // Do commenting
            var currentLine = rctx.textInRange(lctx.contentRange);
            var ind = indentation(currentLine);
            var newline = "" + startIndentation + lcom + " " + currentLine.substr(startIndentation.length);
            rval = newline;
         }
         
         var oldlen = lctx.contentRange.length;
         var newlen = rval.length;
         runningdelta += newlen - oldlen;
         
         // sputs("oldlen: " + String(oldlen));
         // sputs("newlen: " + String(newlen));
         // sputs("rundel: " + String(runningdelta));
         
         if (runningdelta !== 0 && lctx.conventionalRange.location <= selrange.location && selrange.location < lctx.conventionalRange.location + lctx.conventionalRange.length) {
            if (newselrange.location + runningdelta >= 0) {
               newselrange.location += runningdelta;
            }
            else {
               newselrange.location = 0;
            }
            runningdelta = 0;
         }
         else if (runningdelta !== 0 && lctx.conventionalRange.location <= selrange.location + selrange.length && selrange.location + selrange.length <= lctx.conventionalRange.location + lctx.conventionalRange.length) {
            if (newselrange.length + runningdelta >= 0) {
               newselrange.length += runningdelta;
            }
            else {
               newselrange.length = 0;
            }
            runningdelta = 0;
         }
         
         return rval;
         
      }, lineRange);
      /*
      if (oldSelection.length + offset < 0)
      {
         if (oldSelection.location + offset < 0)
            oldSelection.location = 0;
         else
            oldSelection.location += offset;
      }
      else
         oldSelection.length += offset;
      //rctx.selection = oldSelection;
      */
      // sputs("newsel.loc: " + String(newselrange.location));
      // sputs("newsel.len: " + String(newselrange.length));
      if (!allcom || !anycom)
         rctx.selection = newselrange;
   });
}

function comment_block_like_line(blcom, brcom) {
   
}
function comment_block_inner(blcom, brcom) {
   
   Recipe.run(function(rctx) {
      
      // Insert blcom at the left of the selection
      if (rctx.selection.length > 0) {
         var newsel = rctx.selection;
         newsel.length += blcom.length + brcom.length + 2;
         rctx.replaceTextInRange(rctx.selection, blcom + " " + rctx.textInRange(rctx.selection) + " " + brcom);
      }
      else {
         var newsel = rctx.selection;
         newsel.length = 0;
         newsel.location += blcom.length + 1;
         rctx.replaceTextInRange(rctx.selection, blcom + "  " + brcom);
         rctx.selection = newsel;
      }
      
      // Insert brcom at the right of the selection
      //sputs(blcom);
      //sputs(brcom);
      //sputs(rctx.selection.location + blcom.length + rctx.selection.length);
      //rctx.insertTextAtLocation(rctx.selection.location, blcom);
      // rctx.insertTextAtLocation(rctx.selection.location + blcom.length + rctx.selection.length, brcom);
      // rctx.insertTextAtLocation(2, "--!>");
              
   });
}


function comment_line() {
   comment(true);
}
function comment_block() {
   comment(false);
}

function comment(preferLine) {
   // Get the line comment character
   var metadatas = Language.metadata();
   var selctx = Context.context();
   
   var metadata = null;
   var lcom = null;
   var blcom = null;
   var brcom = null;
   
   // We have to do HTML last
   var htmlitem = metadatas['html'];

   for (var key in metadatas) {
      
      // Does this key match the scope
      if (key !== 'html' && selector_matches(key, selctx)) {
         metadata = metadatas[key];
         break;
      }
   }
   if (metadata === null && selector_matches('html', selctx)) {
      metadata = metadatas['html'];
   }
   
   if (metadata !== null) {
      lcom = metadata["comment.line"];
      blcom = metadata["comment.block.start"];
      brcom = metadata["comment.block.end"];
   }
   // sputs(lcom);
   // sputs(blcom);
   // sputs(blcom);
   
   
   if (lcom == null) {
      preferLine = false;
   }
   else if (blcom == null || brcom == null) {
      preferLine = true;
   }
   //else if (lcom == null && (blcom == null || brcom == null)) {
   //   return;
   //}
   
   if (lcom === null || typeof lcom === "undefined") {
      lcom = "//";
   }
   if (blcom === null || typeof blcom === "undefined") {
      blcom = "/*";
   }
   if (brcom === null || typeof brcom === "undefined") {
      brcom = "*/";
   }
   
   if (preferLine) {
      comment_line_inner(lcom);
   }
   else {
      comment_block_inner(blcom, brcom);
   }
}